# Description:  Table reorganization for WP-Table Reloaded Wordpress plugin
# Functions:    Change number of columns, sort rows with regular expression
# Copyright(c): 2011, DSmidge, http://blog.slo-host.com/
# License:      GNU GPL, http://www.gnu.org/licenses/gpl-3.0.txt

# Imports
import os
import re
import sys
# Check argument corectness
if len(sys.argv) != 5:
	print("Use three parameters: file.html number_of_cols rendering regex_for_sorting")
	print("Rendering: H       -> horizontal;")
	print("           V       -> vertical.")
	print("Sorting:   0       -> do not sort;")
	print("           \"RegEx\" -> sort by regular expression;")
	print("           (a)     -> sort by text in first <a> tag.")
	print("This program comes with ABSOLUTELY NO WARRANTY.")
	sys.exit()
# Read input variables
fnam = sys.argv[1]
if not os.path.isfile(fnam):
	sys.exit('File "' + fnam + '" does not exist!')
cols = sys.argv[2]
if cols.isdigit():
	cols = int(cols)
	if cols == 0:
		sys.exit('Parameter number_of_cols must be greater then zero!')
else:
	sys.exit('Parameter number_of_cols must be a digit!')
rend = sys.argv[3]
if rend != 'H' and rend != 'V':
	sys.exit('Parameter rendering is not valid!')
sort = sys.argv[4]
if sort == '(a)':
	sort = '(>[^<>]+</a>)'
# Read input file into list
with open(fnam, 'r') as file:
	oldc = file.readlines()
# Remove new lines between html tags
newc = []
oldcLen = len(oldc)
for line in range(oldcLen):
	strp = oldc[line].strip()
	if strp[:7] == '<table>' or strp[:4] == '<tr>' or strp[:4] == '<td>' \
	or strp[:8] == '</table>' or strp[:5] == '</tr>' :
		newc.append(strp)
	else:
		newc[len(newc)-1] += strp
oldc = newc
# Prepare lines in file for sorting
newc = []
oldcLen = len(oldc)
for line in range(oldcLen):
	# This is column record; store it in a list
	if oldc[line].strip()[:4] == '<td>' and oldc[line].strip() != '<td></td>':
		# Split the line by regex parameter
		splt = re.split(sort, oldc[line].strip(), 1)
		# List has to have at least two elements
		if len(splt) >= 2:
			newc.append(splt)
		else:
			splt.append('')
			newc.append(splt)
# Sorting with regex, use the 2nd element
if sort != "0":
	newc = sorted(newc, key=lambda re: re[1].lower())
# Rearange elements for vertical rendering
newcLen = len(newc)
if rend == 'V':
	newcVer = []
	# Fill extra rows to form a cols*rows box
	newcVerColMod = newcLen%cols
	if newcVerColMod > 0:
		# Append the required number of empty elements
		for elem in range (cols - newcVerColMod):
			newc.append(['', ''])
		# Recalculate the number of elements
		newcLen = len(newc)
	# Calculate elements per column
	newcVerColLen = int(newcLen/cols)
	# Generate new list with vertical order
	for elem in range(newcLen):
		# Calculate the possition of the element, that should be read
		newcVerEle = int(elem/cols) + int(elem%cols)*newcVerColLen
		# Store the entry in a new list
		newcVer.append(newc[newcVerEle])
	# Replace the horizontal result with vertical one
	newc = newcVer
# Generate new html file
fnam = fnam.replace('.html', '.new.html')
with open(fnam, 'w') as file:
	# Add header
	file.write('<table>\n\t<tr>\n')
	# Add sorted elements
	for elem in range(newcLen):
		# Add row break
		if elem != 0 and elem % cols == 0:
			file.write('\t</tr>\n\t<tr>\n')
		# Add column
		file.write('\t\t' + ''.join(newc[elem]) + '\n')
	# Add missing cells
	cellLen = cols - (newcLen % cols)
	if cellLen != cols:
		for cell in range(cellLen):
			file.write('\t\t<td></td>\n')
	# Add footer
	file.write('\t</tr>\n</table>\n')
# Finished
print('Output was saved in "' + fnam + '".')
